/**************************************************************************************************
	$Header: /pub/cvsroot/mydns/src/mydns/udp.c,v 1.26 2003/08/03 17:01:21 bboy Exp $

	Copyright (C) 2002, 2003  Don Moore <bboy@bboy.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at Your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************************************/

#include "named.h"


/**************************************************************************************************
	READ_UDP_QUERY
	Returns 0 on success (a task was added), -1 on failure.
**************************************************************************************************/
int
read_udp_query(int fd)
{
	struct sockaddr_in addr;
	char		in[DNS_MAXPACKETLEN_UDP];
	int		addrlen, len;
	TASK		*t;

	/* Read message */
	addrlen = sizeof(addr);
	memset(&in, 0, sizeof(in));
	if ((len = recvfrom(fd, &in, sizeof(in), 0, (struct sockaddr *)&addr, &addrlen)) <= 0)
		return (-1);

#if DEBUG_ENABLED
	Debug("%s: %d %s", inet_ntoa(addr.sin_addr), len, _("UDP octets in"));
#endif

	if (!(t = task_init(NEED_ANSWER, fd, SOCK_DGRAM, &addr)))
		return (-1);
	return new_task(t, in, len);
}
/*--- read_udp_query() --------------------------------------------------------------------------*/


/**************************************************************************************************
	WRITE_UDP_REPLY
**************************************************************************************************/
int
write_udp_reply(TASK *t)
{
	int rv;

	if ((rv = sendto(t->fd, t->reply, t->replylen, MSG_DONTWAIT,
					(struct sockaddr *)&t->addr, sizeof(t->addr))) < 0)
	{
		Warn("%s: %s", desctask(t), _("error writing reply"));
		dequeue(Tasks, t);
		return (-1);
	}

	/* Task complete; dequeue */
#if DEBUG_ENABLED
	Debug("%s: WRITE %u UDP octets (id %u)", desctask(t), t->replylen, t->id);
#endif
	dequeue(Tasks, t);

	return (0);
}
/*--- write_udp_reply() -------------------------------------------------------------------------*/

/* vi:set ts=3: */
/* NEED_PO */
